# Part of the pfile-tools package
# Copyright (c) 2012, Board of Regents of the University of Wisconsin
# Written by Nathan Vack <njvack@wisc.edu>
# A library for performing anonymization of GE p-file headers.

from collections import namedtuple

import struct_utils
import logging
logger = logging.getLogger(__name__)

# option_name and description are for the command-line tool
AnonEntry = namedtuple("AnonEntry",
    ["key", "value", "option_name", "description"])

# Note: key and option_name should be unique in the list.
DEFAULT_LIST = [
    AnonEntry("patient_name",     "ANONYMIZED", "name",        "patient name"),
    AnonEntry("patient_id",       "ANONYMIZED", "id",          "patient ID"),
    AnonEntry("patient_name_2",   "ANONYMIZED", "name_2",      "patient name 2"),
    AnonEntry("patient_id_2",     "ANONYMIZED", "id_2",        "patient ID 2"),
    AnonEntry("date_of_birth",    "",           "dateofbirth", "date of birth"),
    AnonEntry("patient_age",      0,            "age",         "age"),
    AnonEntry("patient_weight_g", 0,            "weight",      "weight"),
    AnonEntry("patient_sex",      0,            "sex",         "sex"),
]


class Anonymizer(object):

    def __init__(self, anonymization_list=DEFAULT_LIST):
        """Creates a new Anonymizer.

        Arguments:
        anonymization_list -- a list of objects specifying what attributes
            are to anonymized, and how. Objects must contain "key" and "value"
            properties. Keys are dotted string struct value identifiers.
            See DEFAULT_LIST for examples. Defaults to DEFAULT_LIST.
        """
        self.anonymization_list = anonymization_list

    def anonymize(self, header):
        """Runs through self.anonymization_list and anonymizes the header
        in place.

        Arguments:
        header -- a pfile header. NOTE: This structure will be modified
            in place!
        """
        logger.debug("Working with a %s" % (type(header)))
        for entry in self.anonymization_list:
            if struct_utils.has_struct_value(header, entry.key):
                logger.debug("Setting %s to %s" % (entry.key, entry.value))
                struct_utils.set_struct_value(header, entry.key, entry.value)
            else:
                logger.debug("%s not found in header" % entry.key)



#!/usr/bin/env python
# Part of the pfile-tools package
# Copyright (c) 2012, Board of Regents of the University of Wisconsin
# Written by Nathan Vack <njvack@wisc.edu>
#
# An executable script to dump the data from a p-file.

import sys
import optparse
import csv

import pfile_tools
from pfile_tools import headers, struct_utils


def build_option_parser():
    revision_opt_strs = ", ".join([str(r) for r in headers.known_revisions()])
    p = optparse.OptionParser(
        usage="usage: %prog [OPTIONS] pfile",
        description="Dumps header information from a GE P-file",
        version="%prog "+pfile_tools.VERSION)
    p.add_option(
        "-r", "--revision", action="store", choices=headers.known_revisions(),
        help="Force a header revision (available: %s)" % revision_opt_strs)
    p.add_option(
        "--offsets", action="store_true", default=False, dest="offsets",
        help="Show offsets to data elements")
    p.add_option(
        "--sizes", action="store_true", default=False, dest="sizes",
        help="Show data element sizes")
    p.add_option(
        "--show-padding", action="store_true", default=False, dest="padding",
        help="Print unknown 'padding' elements")
    p.add_option(
        "--separator", action="store", default="\t",
        help="Output field separator (default: \\t)")
    return p


def header_columns(opts):
    headers = ["field", "value"]
    if opts.offsets:
        headers.append("offset")
    if opts.sizes:
        headers.append("size")
    return headers


def to_list(info, opts):
    out = [info.label, str(info.value)]
    if opts.offsets:
        out.append("%#x" % (info.offset))
    if opts.sizes:
        out.append(str(info.size))
    return out


def main():
    parser = build_option_parser()
    opts, args = parser.parse_args()
    if len(args) < 1:
        parser.error("Must specify a p-file.")
    rev = opts.revision
    ph = headers.Pfile.from_file(args[0], force_revision=rev)
    dumped = struct_utils.dump_struct(ph.header)
    writer = csv.writer(sys.stdout, delimiter=opts.separator)
    writer.writerow(header_columns(opts))
    for info in dumped:
        if (info.label.find("pad") == 0) and not opts.padding:
            continue
        writer.writerow(to_list(info, opts))


if __name__ == "__main__":
    main()



# Part of the pfile-tools package
# Copyright (c) 2012, Board of Regents of the University of Wisconsin
# Written by Nathan Vack <njvack@wisc.edu>
# Some utilities for working with ctypes Structures

import ctypes
from collections import namedtuple

StructInfo = namedtuple("StructInfo",
    ["label", "depth", "value", "field_type", "size", "offset"])


def dump_struct(struct, include_structs=False):
    """
    Recursively travels through a ctypes.Structure and returns a list of
    namedtuples, containing label, depth, value, size, and offset.
    If include_structs is true, output will include lines for individual
    structures and their sizes and offsets -- not just non-structure fields.
    """
    output = []
    _dump_struct_rec(struct, output, include_structs)
    return output


def _dump_struct_rec(
    struct, output, include_structs=False, prefix='', depth=0, base_offset=0):
    """
    Internal recursive method for dumping structures.
    Appends to the "output" parameter.
    """
    struct_class = type(struct)
    if include_structs:
        output.append(StructInfo(
            "%s (%s)" % (prefix, struct_class.__name__),
            depth, '', str(struct_class), ctypes.sizeof(struct_class), base_offset))
    for f in struct._fields_:
        name = f[0]
        field_type = f[1]
        field_meta = getattr(struct_class, name)
        field = getattr(struct, name)
        cur_prefix = "%s%s." % (prefix, name)
        field_offset = base_offset + field_meta.offset
        if isinstance(field, ctypes.Structure):
            _dump_struct_rec(field, output, include_structs, cur_prefix,
                depth+1, field_offset)
        else:
            label = prefix+name
            output.append(StructInfo(
                label, depth, field, field_type.__name__, field_meta.size, field_offset))


def set_struct_value(struct, field_name, value):
    """
    Sets a value in a ctypes struct, by dotted struct name
    """
    parts = field_name.split(".")
    sh = struct
    while len(parts) > 1:
        subhead_key = parts[0]
        parts = parts[1:]
        sh = getattr(sh, subhead_key)
    setattr(sh, parts[0], value)


def has_struct_value(struct, field_name):
    parts = field_name.split(".")
    sh = struct
    while len(parts) > 1:
        subhead_key = parts[0]
        parts = parts[1:]
        if not hasattr(sh, subhead_key):
            return False
        sh = getattr(sh, subhead_key)
    return hasattr(sh, field_name)


#!/usr/bin/env python
# Part of the pfile-tools package
# Copyright (c) 2012, Board of Regents of the University of Wisconsin
# Written by Nathan Vack <njvack@wisc.edu>
#
# A script to strip the identifying information from a GE p-file.

import optparse
import sys
import shutil
import logging
logger = logging.getLogger(__name__)

import pfile_tools
from pfile_tools import headers, anonymizer


def build_option_parser(anonymization_list):
    revision_opt_strs = ", ".join([str(r) for r in headers.known_revisions()])
    p = optparse.OptionParser(
        usage="usage: %prog [OPTIONS] pfile pfile_out",
        description="Removes personally-identifying information from a GE P-file",
        version="%prog "+pfile_tools.VERSION)
    p.add_option(
        "-r", "--revision", action="store", choices=headers.known_revisions(),
        help="Force a header revision (available: %s)" % revision_opt_strs)
    p.add_option("--inplace", action="store_true",
        help="Edit file in-place. Ignores pfile_out.")
    p.add_option("-v", "--verbose", action="store_true",
        help="Print lots of extra debugging.")
    group = optparse.OptionGroup(p, "Anonymization options")
    for entry in anonymization_list:
        group.add_option(
            "--"+entry.option_name, action="store", type="choice",
            choices=["yes", "no"], default="yes", metavar="yes/no",
            help="Set %s to %r (yes)" % (entry.description, entry.value))
    p.add_option_group(group)
    return p


def filter_anonymization_list(l, options):
    filtered = []
    for entry in l:
        opt_val = getattr(options, entry.option_name)
        if opt_val == "yes":
            logger.debug("Anonymizing %s" % entry.option_name)
            filtered.append(entry)
        else:
            logger.debug("Not anonymizing %s" % entry.option_name)
    return filtered


def setup_logger(options):
    log_level = logging.ERROR
    if options.verbose:
        log_level = logging.DEBUG
    logging.basicConfig(level=log_level)


def setup_files(options, args):
    pfile_in = args[0]
    if not options.inplace:
        pfile_out = args[1]
    else:
        logger.debug("Editing %s in place" % (pfile_in))
        pfile_out = pfile_in
    return (pfile_in, pfile_out)


def read_pfile(pfile_in, options):
    rev = options.revision
    logger.debug("Reading %s, revision %s" % (pfile_in, rev))
    return headers.Pfile.from_file(pfile_in, force_revision=rev)


def write_header(f, ph):
    """
    Write a pfle header to a file (file or filename)
    """
    if hasattr(f, 'seek'):
        return _write_header_inner(f, ph)
    else:
        with open(f, "r+b") as infile:
            return _write_header_inner(infile, ph)


def _write_header_inner(f, ph):
    f.write(ph)


def main():
    parser = build_option_parser(anonymizer.DEFAULT_LIST)
    (options, args) = parser.parse_args()
    if len(args) < 2 and not options.inplace:
        parser.error("Both pfile_in and pfile_out are required")
    if len(args) < 1:
        parser.error("pfile_in is required")
    setup_logger(options)
    pfile_in, pfile_out = setup_files(options, args)
    pfile = read_pfile(pfile_in, options)

    anon_list = filter_anonymization_list(anonymizer.DEFAULT_LIST, options)
    a = anonymizer.Anonymizer(anon_list)
    a.anonymize(pfile.header)

    if not options.inplace:
        logger.debug("Copying %s to %s" % (pfile_in, pfile_out))
        shutil.copyfile(pfile_in, pfile_out)
    write_header(pfile_out, pfile.header)


if __name__ == "__main__":
    main()



# Part of the pfile-tools package
# Copyright (c) 2012, Board of Regents of the University of Wisconsin
# Written by Nathan Vack <njvack@wisc.edu>
#
# Contains the ctypes Structure for a GE P-file header.

from ctypes import *
import datetime


def REVISIONS():
    return {
        '16'    : R16PfileHeader,
        '20.006': R20_006PfileHeader,
        '20.007': R20_007PfileHeader
    }


def format_short_float(f):
    """
    Formats 16.00 to 16 and 20.006000 to 20.006
    """
    return ("%.5f" % f).rstrip("0").rstrip(".")


def known_revisions():
    return [x for x in sorted(REVISIONS().keys())]


class Pfile(object):
    """
    The wrapper class for all manner of pfile header readin' structs.
    Really, only the one for now. Who knows, maybe ever?
    """

    def __init__(self, header, revision):
        self.header = header
        self.revision = revision
        for f in self.header._fields_:
            # Copy all the fields into this class.
            name = f[0]
            setattr(self, name, getattr(self.header, name))

    @property
    def exam_datetime(self):
        return datetime.datetime.utcfromtimestamp(self.exam_timestamp)

    @property
    def series_datetime(self):
        return datetime.datetime.utcfromtimestamp(self.series_timestamp)

    @classmethod
    def from_file(cls, infile, force_revision=None):
        filelike = infile
        if not hasattr('seek', filelike):
            filelike = open(filelike, 'rb')
        revision = force_revision or format_short_float(cls._major_revision(filelike))
        filelike.seek(0)
        rev_hash = REVISIONS()
        header_cls = rev_hash.get(revision)
        if header_cls is None:
            raise UnknownRevision("No header found for revision %s" % revision)
        header = header_cls()
        filelike.readinto(header)
        return cls(header, revision)

    @classmethod
    def _major_revision(cls, filelike):
        rnh = RevisionNum()
        filelike.seek(0)
        filelike.readinto(rnh)
        return rnh.revision


class UnknownRevision(RuntimeError):
    pass


class RevisionNum(LittleEndianStructure):

    _pack_ = 1

    _fields_ = [
        ('revision', c_float)]


class R16PfileHeader(LittleEndianStructure):

    _pack_ = 1

    _fields_ = [
        ('revision', c_float),
        ('pad_0', c_char * 12),
        ('scan_date_str', c_char * 10),
        ('scan_time_str', c_char * 8),
        ('pad_1', c_char * 30),
        ('pass_count', c_short),
        ('pad_2', c_char * 2),
        ('slice_count', c_ushort),
        ('echo_count', c_short),
        ('pad_3', c_char * 2),
        ('frame_count', c_short),
        ('pad_4', c_char * 4),
        ('frame_size', c_ushort),
        ('pad_5', c_char * 20),
        ('acq_x_res', c_ushort),
        ('acq_y_Res', c_short),
        ('recon_x_res', c_short),
        ('recon_y_res', c_short),
        ('image_size', c_short),
        ('recon_z_res', c_int),
        ('pad_6', c_char * 100),
        ('rh_user_0', c_float),
        ('rh_user_1', c_float),
        ('rh_user_2', c_float),
        ('rh_user_3', c_float),
        ('rh_user_4', c_float),
        ('rh_user_5', c_float),
        ('rh_user_6', c_float),
        ('rh_user_7', c_float),
        ('rh_user_8', c_float),
        ('rh_user_9', c_float),
        ('rh_user_10', c_float),
        ('rh_user_11', c_float),
        ('rh_user_12', c_float),
        ('rh_user_13', c_float),
        ('rh_user_14', c_float),
        ('rh_user_15', c_float),
        ('rh_user_16', c_float),
        ('rh_user_17', c_float),
        ('rh_user_18', c_float),
        ('rh_user_19', c_float),
        ('pad_7', c_char * 704),
        ('rh_user_20', c_float),
        ('rh_user_21', c_float),
        ('rh_user_22', c_float),
        ('rh_user_23', c_float),
        ('rh_user_24', c_float),
        ('rh_user_25', c_float),
        ('rh_user_26', c_float),
        ('rh_user_27', c_float),
        ('rh_user_28', c_float),
        ('rh_user_29', c_float),
        ('rh_user_30', c_float),
        ('rh_user_31', c_float),
        ('rh_user_32', c_float),
        ('rh_user_33', c_float),
        ('rh_user_34', c_float),
        ('rh_user_35', c_float),
        ('rh_user_36', c_float),
        ('rh_user_37', c_float),
        ('rh_user_38', c_float),
        ('rh_user_39', c_float),
        ('rh_user_40', c_float),
        ('rh_user_41', c_float),
        ('rh_user_42', c_float),
        ('rh_user_43', c_float),
        ('rh_user_44', c_float),
        ('rh_user_45', c_float),
        ('rh_user_46', c_float),
        ('rh_user_47', c_float),
        ('rh_user_48', c_float),
        ('pad_8', c_char * 528),
        ('bandwidth', c_float),
        ('pad_9', c_char * 12),
        ('data_size', c_ulong),
        ('ssp_save', c_ulong),
        ('uda_save', c_ulong),
        ('pad_10', c_char * 137214),
        ('aps_r1', c_int),
        ('aps_r2', c_int),
        ('aps_tg', c_int),
        ('aps_frequency', c_uint),
        ('scale_i', c_float),
        ('scale_q', c_float),
        ('pad_11', c_char * 276),
        ('x_shim', c_short),
        ('y_shim', c_short),
        ('z_shim', c_short),
        ('recon_enabled', c_short),
        ('pad_12', c_char * 1774),
        ('magnet_strength', c_int),
        ('patient_weight_g', c_int),
        ('exam_timestamp', c_int),
        ('pad_13', c_char * 52),
        ('exam_number', c_ushort),
        ('pad_14', c_char * 18),
        ('patient_age', c_short),
        ('pad_15', c_char * 2),
        ('patient_sex', c_short),
        ('pad_16', c_char * 2),
        ('patient_trauma', c_short),
        ('pad_17', c_char * 2),
        ('study_status', c_short),
        ('pad_18', c_char * 166),
        ('exam_description', c_char * 65),
        ('exam_type', c_char * 3),
        ('system_id', c_char * 9),
        ('pad_20', c_char * 14),
        ('hospital_name', c_char * 33),
        ('patient_id_2', c_char * 13),
        ('patient_name_2', c_char * 25),
        ('service_id', c_char * 16),
        ('pad_22', c_char * 124),
        ('patient_name', c_char * 65),
        ('patient_id', c_char * 65),
        ('req_num', c_char * 17),
        ('date_of_birth', c_char * 9),
        ('pad_23', c_char * 492),
        ('series_number', c_short),
        ('pad_26', c_char * 122),
        ('series_description', c_char * 65),
        ('pad_27', c_char * 21),
        ('protocol', c_char * 25),
        ('start_ras', c_char),
        ('end_ras', c_char),
        ('pad_28', c_char * 1541),
        ('x_field_of_view', c_float),
        ('y_field_of_view', c_float),
        ('scan_duration', c_float),
        ('z_thickness', c_float),
        ('pad_29', c_char * 36),
        ('op_user_0', c_float),
        ('op_user_1', c_float),
        ('op_user_2', c_float),
        ('op_user_3', c_float),
        ('op_user_4', c_float),
        ('op_user_5', c_float),
        ('op_user_6', c_float),
        ('op_user_7', c_float),
        ('op_user_8', c_float),
        ('op_user_9', c_float),
        ('op_user_10', c_float),
        ('op_user_11', c_float),
        ('op_user_12', c_float),
        ('op_user_13', c_float),
        ('op_user_14', c_float),
        ('op_user_15', c_float),
        ('op_user_16', c_float),
        ('op_user_17', c_float),
        ('op_user_18', c_float),
        ('op_user_19', c_float),
        ('op_user_20', c_float),
        ('op_user_21', c_float),
        ('op_user_22', c_float),
        ('pad_30', c_char * 8),
        ('op_user_23', c_float),
        ('op_user_24', c_float),
        ('pad_31', c_char * 60),
        ('op_user_25', c_float),
        ('op_user_26', c_float),
        ('op_user_27', c_float),
        ('op_user_28', c_float),
        ('op_user_29', c_float),
        ('op_user_30', c_float),
        ('op_user_31', c_float),
        ('op_user_32', c_float),
        ('op_user_33', c_float),
        ('op_user_34', c_float),
        ('op_user_35', c_float),
        ('op_user_36', c_float),
        ('op_user_37', c_float),
        ('op_user_38', c_float),
        ('op_user_39', c_float),
        ('op_user_40', c_float),
        ('op_user_41', c_float),
        ('op_user_42', c_float),
        ('op_user_43', c_float),
        ('op_user_44', c_float),
        ('op_user_45', c_float),
        ('op_user_46', c_float),
        ('op_user_47', c_float),
        ('op_user_48', c_float),
        ('pad_32', c_char * 60),
        ('x_dim', c_float),
        ('y_dim', c_float),
        ('x_size', c_float),
        ('y_size', c_float),
        ('r_center', c_float),
        ('a_center', c_float),
        ('s_center', c_float),
        ('r_norm', c_float),
        ('a_norm', c_float),
        ('s_norm', c_float),
        ('pad_33', c_char * 232),
        ('tr', c_int),
        ('ti', c_int),
        ('te', c_int),
        ('pad_34', c_char * 548),
        ('psd_name', c_char * 33),
        ('pad_36', c_char * 84),
        ('coil_name', c_char * 17),
        ('pad_37', c_char * 115),
        ('long_coil_name', c_char * 24)]


class R20_006PfileHeader(LittleEndianStructure):

    _pack_ = 1

    _fields_ = [
        ('revision', c_float),
        ('pad_0', c_char * 12),
        ('scan_date_str', c_char * 10),
        ('scan_time_str', c_char * 8),
        ('pad_1', c_char * 30),
        ('pass_count', c_short),
        ('pad_2', c_char * 2),
        ('slice_count', c_ushort),
        ('echo_count', c_short),
        ('pad_3', c_char * 2),
        ('frame_count', c_short),
        ('pad_4', c_char * 4),
        ('frame_size', c_ushort),
        ('pad_5', c_char * 20),
        ('acq_x_res', c_ushort),
        ('acq_y_Res', c_short),
        ('recon_x_res', c_short),
        ('recon_y_res', c_short),
        ('image_size', c_short),
        ('recon_z_res', c_int),
        ('pad_6', c_char * 100),
        ('rh_user_0', c_float),
        ('rh_user_1', c_float),
        ('rh_user_2', c_float),
        ('rh_user_3', c_float),
        ('rh_user_4', c_float),
        ('rh_user_5', c_float),
        ('rh_user_6', c_float),
        ('rh_user_7', c_float),
        ('rh_user_8', c_float),
        ('rh_user_9', c_float),
        ('rh_user_10', c_float),
        ('rh_user_11', c_float),
        ('rh_user_12', c_float),
        ('rh_user_13', c_float),
        ('rh_user_14', c_float),
        ('rh_user_15', c_float),
        ('rh_user_16', c_float),
        ('rh_user_17', c_float),
        ('rh_user_18', c_float),
        ('rh_user_19', c_float),
        ('pad_7', c_char * 704),
        ('rh_user_20', c_float),
        ('rh_user_21', c_float),
        ('rh_user_22', c_float),
        ('rh_user_23', c_float),
        ('rh_user_24', c_float),
        ('rh_user_25', c_float),
        ('rh_user_26', c_float),
        ('rh_user_27', c_float),
        ('rh_user_28', c_float),
        ('rh_user_29', c_float),
        ('rh_user_30', c_float),
        ('rh_user_31', c_float),
        ('rh_user_32', c_float),
        ('rh_user_33', c_float),
        ('rh_user_34', c_float),
        ('rh_user_35', c_float),
        ('rh_user_36', c_float),
        ('rh_user_37', c_float),
        ('rh_user_38', c_float),
        ('rh_user_39', c_float),
        ('rh_user_40', c_float),
        ('rh_user_41', c_float),
        ('rh_user_42', c_float),
        ('rh_user_43', c_float),
        ('rh_user_44', c_float),
        ('rh_user_45', c_float),
        ('rh_user_46', c_float),
        ('rh_user_47', c_float),
        ('rh_user_48', c_float),
        ('pad_8', c_char * 528),
        ('bandwidth', c_float),
        ('pad_9', c_char * 12),
        ('data_size', c_ulong),
        ('ssp_save', c_ulong),
        ('uda_save', c_ulong),
        ('pad_10', c_char * 139656),
        ('aps_r1', c_int),
        ('aps_r2', c_int),
        ('aps_tg', c_int),
        ('aps_frequency', c_uint),
        ('scale_i', c_float),
        ('scale_q', c_float),
        ('pad_11', c_char * 276),
        ('x_shim', c_short),
        ('y_shim', c_short),
        ('z_shim', c_short),
        ('recon_enabled', c_short),
        ('pad_12', c_char * 1744),
        ('magnet_strength', c_int),
        ('patient_weight_g', c_int),
        ('exam_timestamp', c_int),
        ('pad_13', c_char * 112),
        ('exam_number', c_ushort),
        ('pad_14', c_char * 18),
        ('patient_age', c_short),
        ('pad_15', c_char * 2),
        ('patient_sex', c_short),
        ('pad_16', c_char * 2),
        ('patient_trauma', c_short),
        ('pad_17', c_char * 2),
        ('study_status', c_short),
        ('pad_18', c_char * 70),
        ('history', c_char * 257),
        ('pad_19', c_char * 195),
        ('exam_description', c_char * 65),
        ('exam_type', c_char * 3),
        ('system_id', c_char * 9),
        ('pad_20', c_char * 14),
        ('hospital_name', c_char * 33),
        ('pad_21', c_char * 24),
        ('service_id', c_char * 16),
        ('pad_22', c_char * 100),
        ('patient_name', c_char * 65),
        ('patient_id', c_char * 65),
        ('req_num', c_char * 17),
        ('date_of_birth', c_char * 9),
        ('pad_23', c_char * 560),
        ('start_location', c_float),
        ('end_location', c_float),
        ('pad_24', c_char * 352),
        ('series_timestamp', c_int),
        ('pad_25', c_char * 206),
        ('series_number', c_short),
        ('pad_26', c_char * 138),
        ('series_description', c_char * 65),
        ('pad_27', c_char * 21),
        ('protocol', c_char * 25),
        ('start_ras', c_char),
        ('end_ras', c_char),
        ('pad_28', c_char * 1769),
        ('x_field_of_view', c_float),
        ('y_field_of_view', c_float),
        ('scan_duration', c_float),
        ('z_thickness', c_float),
        ('pad_29', c_char * 36),
        ('op_user_0', c_float),
        ('op_user_1', c_float),
        ('op_user_2', c_float),
        ('op_user_3', c_float),
        ('op_user_4', c_float),
        ('op_user_5', c_float),
        ('op_user_6', c_float),
        ('op_user_7', c_float),
        ('op_user_8', c_float),
        ('op_user_9', c_float),
        ('op_user_10', c_float),
        ('op_user_11', c_float),
        ('op_user_12', c_float),
        ('op_user_13', c_float),
        ('op_user_14', c_float),
        ('op_user_15', c_float),
        ('op_user_16', c_float),
        ('op_user_17', c_float),
        ('op_user_18', c_float),
        ('op_user_19', c_float),
        ('op_user_20', c_float),
        ('op_user_21', c_float),
        ('op_user_22', c_float),
        ('pad_30', c_char * 8),
        ('op_user_23', c_float),
        ('op_user_24', c_float),
        ('pad_31', c_char * 60),
        ('op_user_25', c_float),
        ('op_user_26', c_float),
        ('op_user_27', c_float),
        ('op_user_28', c_float),
        ('op_user_29', c_float),
        ('op_user_30', c_float),
        ('op_user_31', c_float),
        ('op_user_32', c_float),
        ('op_user_33', c_float),
        ('op_user_34', c_float),
        ('op_user_35', c_float),
        ('op_user_36', c_float),
        ('op_user_37', c_float),
        ('op_user_38', c_float),
        ('op_user_39', c_float),
        ('op_user_40', c_float),
        ('op_user_41', c_float),
        ('op_user_42', c_float),
        ('op_user_43', c_float),
        ('op_user_44', c_float),
        ('op_user_45', c_float),
        ('op_user_46', c_float),
        ('op_user_47', c_float),
        ('op_user_48', c_float),
        ('pad_32', c_char * 60),
        ('x_dim', c_float),
        ('y_dim', c_float),
        ('x_size', c_float),
        ('y_size', c_float),
        ('r_center', c_float),
        ('a_center', c_float),
        ('s_center', c_float),
        ('r_norm', c_float),
        ('a_norm', c_float),
        ('s_norm', c_float),
        ('pad_33', c_char * 336),
        ('tr', c_int),
        ('ti', c_int),
        ('te', c_int),
        ('pad_34', c_char * 432),
        ('frequency_direction', c_short),
        ('pad_35', c_char * 130),
        ('psd_name', c_char * 33),
        ('pad_36', c_char * 84),
        ('coil_name', c_char * 17),
        ('pad_37', c_char * 115),
        ('long_coil_name', c_char * 24),
        ('pad_38', c_char * 543)]

class R20_007PfileHeader(LittleEndianStructure):

    _pack_ = 1

    _fields_ = [
        ('revision', c_float),
        ('pad_0', c_char * 12),
        ('scan_date_str', c_char * 10),
        ('scan_time_str', c_char * 8),
        ('pad_1', c_char * 30),
        ('pass_count', c_short),
        ('pad_2', c_char * 2),
        ('slice_count', c_ushort),
        ('echo_count', c_short),
        ('pad_3', c_char * 2),
        ('frame_count', c_short),
        ('pad_4', c_char * 4),
        ('frame_size', c_ushort),
        ('pad_5', c_char * 20),
        ('acq_x_res', c_ushort),
        ('acq_y_Res', c_short),
        ('recon_x_res', c_short),
        ('recon_y_res', c_short),
        ('image_size', c_short),
        ('recon_z_res', c_int),
        ('pad_6', c_char * 100),
        ('rh_user_0', c_float),
        ('rh_user_1', c_float),
        ('rh_user_2', c_float),
        ('rh_user_3', c_float),
        ('rh_user_4', c_float),
        ('rh_user_5', c_float),
        ('rh_user_6', c_float),
        ('rh_user_7', c_float),
        ('rh_user_8', c_float),
        ('rh_user_9', c_float),
        ('rh_user_10', c_float),
        ('rh_user_11', c_float),
        ('rh_user_12', c_float),
        ('rh_user_13', c_float),
        ('rh_user_14', c_float),
        ('rh_user_15', c_float),
        ('rh_user_16', c_float),
        ('rh_user_17', c_float),
        ('rh_user_18', c_float),
        ('rh_user_19', c_float),
        ('pad_7', c_char * 704),
        ('rh_user_20', c_float),
        ('rh_user_21', c_float),
        ('rh_user_22', c_float),
        ('rh_user_23', c_float),
        ('rh_user_24', c_float),
        ('rh_user_25', c_float),
        ('rh_user_26', c_float),
        ('rh_user_27', c_float),
        ('rh_user_28', c_float),
        ('rh_user_29', c_float),
        ('rh_user_30', c_float),
        ('rh_user_31', c_float),
        ('rh_user_32', c_float),
        ('rh_user_33', c_float),
        ('rh_user_34', c_float),
        ('rh_user_35', c_float),
        ('rh_user_36', c_float),
        ('rh_user_37', c_float),
        ('rh_user_38', c_float),
        ('rh_user_39', c_float),
        ('rh_user_40', c_float),
        ('rh_user_41', c_float),
        ('rh_user_42', c_float),
        ('rh_user_43', c_float),
        ('rh_user_44', c_float),
        ('rh_user_45', c_float),
        ('rh_user_46', c_float),
        ('rh_user_47', c_float),
        ('rh_user_48', c_float),
        ('pad_8', c_char * 528),
        ('bandwidth', c_float),
        ('pad_9', c_char * 12),
        ('data_size', c_ulong),
        ('ssp_save', c_ulong),
        ('uda_save', c_ulong),
        ('pad_10', c_char * 139656),
        ('aps_r1', c_int),
        ('aps_r2', c_int),
        ('aps_tg', c_int),
        ('aps_frequency', c_uint),
        ('scale_i', c_float),
        ('scale_q', c_float),
        ('pad_11', c_char * 276),
        ('x_shim', c_short),
        ('y_shim', c_short),
        ('z_shim', c_short),
        ('recon_enabled', c_short),
        ('pad_12', c_char * 1744),
        ('magnet_strength', c_int),
        ('patient_weight_g', c_int),
        ('exam_timestamp', c_int),
        ('pad_13', c_char * 112),
        ('exam_number', c_ushort),
        ('pad_14', c_char * 18),
        ('patient_age', c_short),
        ('pad_15', c_char * 2),
        ('patient_sex', c_short),
        ('pad_16', c_char * 2),
        ('patient_trauma', c_short),
        ('pad_17', c_char * 2),
        ('study_status', c_short),
        ('pad_18', c_char * 70),
        ('history', c_char * 257),
        ('pad_19', c_char * 195),
        ('exam_description', c_char * 65),
        ('exam_type', c_char * 3),
        ('system_id', c_char * 9),
        ('pad_20', c_char * 22),
        ('hospital_name', c_char * 33),
        ('pad_21', c_char * 24),
        ('service_id', c_char * 16),
        ('pad_22', c_char * 100),
        ('patient_name', c_char * 65),
        ('patient_id', c_char * 65),
        ('req_num', c_char * 17),
        ('date_of_birth', c_char * 9),
        ('pad_23', c_char * 552),
        ('start_location', c_float),
        ('end_location', c_float),
        ('pad_24', c_char * 352),
        ('series_timestamp', c_int),
        ('pad_25', c_char * 206),
        ('series_number', c_short),
        ('pad_26', c_char * 138),
        ('series_description', c_char * 65),
        ('pad_27', c_char * 21),
        ('protocol', c_char * 25),
        ('start_ras', c_char),
        ('end_ras', c_char),
        ('pad_28', c_char * 1769),
        ('x_field_of_view', c_float),
        ('y_field_of_view', c_float),
        ('scan_duration', c_float),
        ('z_thickness', c_float),
        ('pad_29', c_char * 36),
        ('op_user_0', c_float),
        ('op_user_1', c_float),
        ('op_user_2', c_float),
        ('op_user_3', c_float),
        ('op_user_4', c_float),
        ('op_user_5', c_float),
        ('op_user_6', c_float),
        ('op_user_7', c_float),
        ('op_user_8', c_float),
        ('op_user_9', c_float),
        ('op_user_10', c_float),
        ('op_user_11', c_float),
        ('op_user_12', c_float),
        ('op_user_13', c_float),
        ('op_user_14', c_float),
        ('op_user_15', c_float),
        ('op_user_16', c_float),
        ('op_user_17', c_float),
        ('op_user_18', c_float),
        ('op_user_19', c_float),
        ('op_user_20', c_float),
        ('op_user_21', c_float),
        ('op_user_22', c_float),
        ('pad_30', c_char * 8),
        ('op_user_23', c_float),
        ('op_user_24', c_float),
        ('pad_31', c_char * 60),
        ('op_user_25', c_float),
        ('op_user_26', c_float),
        ('op_user_27', c_float),
        ('op_user_28', c_float),
        ('op_user_29', c_float),
        ('op_user_30', c_float),
        ('op_user_31', c_float),
        ('op_user_32', c_float),
        ('op_user_33', c_float),
        ('op_user_34', c_float),
        ('op_user_35', c_float),
        ('op_user_36', c_float),
        ('op_user_37', c_float),
        ('op_user_38', c_float),
        ('op_user_39', c_float),
        ('op_user_40', c_float),
        ('op_user_41', c_float),
        ('op_user_42', c_float),
        ('op_user_43', c_float),
        ('op_user_44', c_float),
        ('op_user_45', c_float),
        ('op_user_46', c_float),
        ('op_user_47', c_float),
        ('op_user_48', c_float),
        ('pad_32', c_char * 60),
        ('x_dim', c_float),
        ('y_dim', c_float),
        ('x_size', c_float),
        ('y_size', c_float),
        ('r_center', c_float),
        ('a_center', c_float),
        ('s_center', c_float),
        ('r_norm', c_float),
        ('a_norm', c_float),
        ('s_norm', c_float),
        ('pad_33', c_char * 336),
        ('tr', c_int),
        ('ti', c_int),
        ('te', c_int),
        ('pad_34', c_char * 432),
        ('frequency_direction', c_short),
        ('pad_35', c_char * 130),
        ('psd_name', c_char * 33),
        ('pad_36', c_char * 84),
        ('coil_name', c_char * 17),
        ('pad_37', c_char * 115),
        ('long_coil_name', c_char * 24),
        ('pad_38', c_char * 543)]



#!/usr/bin/env python
from distribute_setup import use_setuptools
use_setuptools()
import setuptools
from setuptools import setup
import os

import pfile_tools


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='pfile-tools',
    version=pfile_tools.VERSION,
    packages=setuptools.find_packages(),
    data_files=[('', ['distribute_setup.py'])],
    license='BSD License',
    long_description=read('README'),
    url="https://github.com/njvack/pfile-tools",
    author="Nathan Vack",
    author_email="njvack@wisc.edu",
    entry_points = {
        'console_scripts': [
            'dump_pfile_header = pfile_tools.scripts.dump_pfile_header:main',
            'anonymize_pfile = pfile_tools.scripts.anonymize_pfile:main'
        ]
    }
)